--------------------
--01 Static Entity--
--------------------

function newStaticEntity(image, x, y, radius)
  local entity = {
    x  = x or 0, -- Coordinates
    y  = y or 0, 
    r  = 0,      -- Rotation
    sx = 1,      -- Scale
    sy = 1, 
  }

  --Defaulted in main.lua
  --image:setFilter('nearest', 'nearest')
   
  entity.width  = image:getWidth()
  entity.height = image:getHeight()
   
  -- NOTE: This is just for BASIC collision detection.
  entity.radius = radius or (entity.width / 2)
   
  entity.ox = entity.width / 2
  entity.oy = entity.height / 2
   
  function entity.update(dt)
    -- For now, this is just a filler.
  end
   
  function entity.draw()
    love.graphics.draw(image, entity.x, entity.y, entity.r, entity.sx, entity.sy, entity.ox, entity.oy)
  end
   
  function entity.setPosition(x, y)
    entity.x = x or entity.x
    entity.y = y or entity.y
  end
  
  function entity.setScale(sx, sy)
    entity.sx = sx or 1
    entity.sy = sy or 1
  end
  
  return entity
end

----------------------
--01 Animated Entity--
----------------------

function newAnimatedEntity(image, x, y, radius)
  local entity = newStaticEntity(image, x, y, radius) -- Yeah, right. Too lazy to type that again.
   
  -- The Basic Animation Things
  local animations = {}
  local current_animation = nil
  entity.frame_timer = 0
  entity.current_frame = 1
  entity.isEndofFrame = false
   
  function entity.addAnimation(id, frame_rate)
    animations[id] = {frame_rate = frame_rate, frames = {}}
  end
   
  -- There are 3 things added in each frame: quad, width, and height
  -- Maybe there's a quad:getWidth()?
  function entity.addAnimationFrames(id, x, y, width, height, number_of_intervals)
    -- Adding the frames can be quite unreadable, so I need these
    local image_width, image_height = image:getWidth(), image:getHeight()
    
    -- From left to right, NOT from top to bottom
    for i = 0, number_of_intervals - 1 do
      local start_x = x + (width * i)
      table.insert(animations[id].frames,{
          quad = love.graphics.newQuad(start_x, y, width, height, image_width, image_height),
          width = width, -- I need the width and height of each frame for centering them in the .draw() function
          height = height
        }
      )
    end
  end
   
  function entity.setAnimation(id)
    if current_animation ~= id then
      current_animation = id
      entity.frame_timer = 0
      entity.current_frame = 1
      entity.isEndofFrame = false
    end
  end
   
  function entity.update(dt)
    -- Detect first if currently set animation has at least 1 frame in it
    if #animations[current_animation].frames > 0 then
      entity.frame_timer = entity.frame_timer + dt
      entity.isEndofFrame = false
      -- Move to next frame according to frame rate
      if entity.frame_timer >= animations[current_animation].frame_rate then
        entity.frame_timer = 0
        entity.current_frame = entity.current_frame + 1
        -- Move back to first frame
        if entity.current_frame > #animations[current_animation].frames then
          entity.isEndofFrame = true
          entity.current_frame = 1
        end
      end

      -- Each frame can have different dimensions
      entity.width = animations[current_animation].frames[entity.current_frame].width
      entity.height = animations[current_animation].frames[entity.current_frame].height
	          
      -- Centering the x and y to the Center of the Image
      entity.ox = entity.width / 2
      entity.oy = entity.height / 2
    end
 end
   
  function entity.draw()
    love.graphics.draw(image, animations[current_animation].frames[entity.current_frame].quad, entity.x, entity.y, entity.r, entity.sx, entity.sy, entity.ox, entity.oy)
  end
   
  return entity
end
