engine = {}

--[[PIECE FUNCTIONS--]]
-- Checks if char is lowercase or not.
function engine.getColor(char)
  if char == ' ' then
    return 0 -- No Color
  elseif char == char:upper() then
    return 1 -- Chu
  elseif char == char:lower() then
    return 2 -- Han
  end
end

--[[BOARD FUNCTIONS--]]
-- A disadvantage of 1D Array for boards is that
-- that it's difficult to visually see the files and ranks.
-- These functions serve that purpose
function engine.getFile(index)
  return ((index - 1) % 9) + 1
end
 -- math.ceil() Rounds Up, math.floor() Round Down  
function engine.getRank(index)
  return math.ceil(index / 9)
end
-- Of course, a reverse function is useful
function engine.getIndex(rank, file)
  return ((rank - 1) * 9) + file
end

function engine.getRepresentation(points)
  local rep = ''
  
  for i = 1, #points do
    rep = rep .. points[i]
  end
  
  return rep
end

function engine.isBikjang(points)
  local bikjang = false
  local chu_general = 0
  local han_general = 0
  for i, v in pairs(points) do
    if v == 'G' then chu_general = i end
    if v == 'g' then han_general = i end
  end

  if engine.getFile(chu_general) == engine.getFile(han_general) then
    local file = engine.getFile(han_general)
    local rank = engine.getRank(han_general) + 1
    local blocking_piece = false
    repeat
      if points[engine.getIndex(rank, file)] ~= ' ' then
        blocking_piece = true
      end
      rank = rank + 1
    until rank == engine.getRank(chu_general) or blocking_piece

    if blocking_piece == false then
      bikjang = true
    end
  end
  
  return bikjang
end

function engine.isJanggun(points, color)
  local general_i = 0
  -- Getting Generals' Indices
  for i = 1, #points do
    if points[i]:upper() == 'G' and engine.getColor(points[i]) == color then
      general_i = i
    end
  end

  local attacked = false
  local b = newBoard(engine.getRepresentation(points), (color%2)+1)
  local attacked = false
  for _, p in pairs(b.pieces) do
    for _, m in pairs(p.moves) do
      if m == general_i then
         attacked = true
      end
    end
  end
  return attacked
end

function engine.isThreefold(move_history, history_position)
  local three_fold = false
  if #move_history >= 9 and history_position == #move_history then
    local rep1 = move_history[#move_history].rep
    local rep2 = move_history[#move_history-4].rep
    local rep3 = move_history[#move_history-8].rep
    
    if rep1 == rep2 and rep2 == rep3 then
      three_fold = true
    end
  end
  
  return three_fold
end

function engine.softEvaluate(points, move_history, color, history_position)
  local evaluation = ''
  local chu_general = 0
  local han_general = 0
  -- Getting Generals' Indices
  for i = 1, #points do
    if points[i] == 'G' then chu_general = i end
    if points[i] == 'g' then han_general = i end
  end

  --#1  PREVIOUS PLAYER'S ACTION:
  --#1a Steps out of the Fortress (Positional Resignation)
  if (color == 2 and chu_general == 79) or (color == 1 and han_general == 12) then
    evaluation = 'Outside General'
  --#1b Accept Bikjang (Positional Draw)
  elseif chu_general == 0 or han_general == 0 then
    evaluation = 'Truce Accepted'
  --#1c Committed Threefold Repitition
  elseif engine.isThreefold(move_history, history_position) then
    evaluation = 'Threefold Repitition'

  --#2  CURRENT PLAYER'S SITUATION caused by other player
  --#2a Bikjang (Facing Generals)
  elseif engine.isBikjang(points) then
    evaluation = 'Truce Offered'
  --#2b Janggun (Check)
  elseif engine.isJanggun(points, color) then
    evaluation = 'Exposed General'
  end

  return evaluation
end

function engine.whoWon(evaluation, pieces, color)
  local winner1 = -1
  local winner2 = -1
  if evaluation == 'Outside General' then
    winner1 = color == 1 and 1 or 2
  elseif evaluation == 'Slain General' then
    winner1 = color == 1 and 2 or 1
  elseif evaluation == 'Truce Accepted' or evaluation == 'Threefold Repitition' or  evaluation == 'Truce Forced' then
    winner1 = 0
    local color1 = 0
    local color2 = 1.5
    for _, v in pairs(pieces) do
      if v.char:upper() ~= 'G' then
        if v.char:upper() == v.char then
          color1 = color1 + v.value
        elseif v.char:lower() == v.char then
          color2 = color2 + v.value
        end
      end
    end
    winner2 = color1 > color2 and 1 or 2
  end
  
  return winner1, winner2
end
