gui_board = { points = {}, flipped = false } -- Points = "Squares" in Chess

local x, y   = 40, 65
local pw, ph = 65, 55 -- Distance between each points (width and height)
local pradius = 4     -- Soldier Cannon Points are distinctly designed
local bsize = 35      -- Border Size (The Board itself, not the points)
local font = love.graphics.newFont()

-- Color of the Board and its lines
local bcolor = {200/255, 160/255, 90/255} -- Light brown
local lcolor = {100/255,  40/255,  0/255} -- Darker brown

-- Inserting the GUI Coordinates (Also a 1-D Array)
for i = 1, 90 do
  gui_board.points[i] = {
    x = x + ((engine.getFile(i) - 1) * pw),
    y = y + ((engine.getRank(i) - 1) * ph)
  }
end
local rank_notations = {}
for i = 1, 10 do
  local filex = gui_board.points[1].x - 25 - (font:getWidth(i) / 2)
  local ranky = gui_board.points[engine.getIndex(i, 1)].y
  local height = font:getHeight(i) / 2
  table.insert(rank_notations, {x = filex, y = ranky - height})
end
local file_notations = {}
for i = 1, 9 do
  local filex = gui_board.points[i].x - (font:getWidth(1) / 2)
  local ranky = gui_board.points[91 - i].y + 22
  table.insert(file_notations, {x = filex, y = ranky})
end

function gui_board.load()
  if gui_board.flipped then gui_board.flip() end
end

function gui_board.draw()
  -- Outer Board
  love.graphics.setFont(font)
  love.graphics.setColor(bcolor)
  love.graphics.rectangle(
    'fill',
    x - bsize,
    y - bsize,
    8 * pw + (bsize * 2), -- 8 = 9 files - 1
    9 * ph + (bsize * 2),  -- 9 = 10 ranks - 1
    6,
    6
  )
  
  -- Inner Board
  love.graphics.setColor(lcolor)
  
  --Vertical Lines
  for i = 1, 9 do
    love.graphics.line(
      gui_board.points[engine.getIndex(1,i)].x, gui_board.points[engine.getIndex(1,i)].y, -- From First Rank
      gui_board.points[engine.getIndex(10,i)].x, gui_board.points[engine.getIndex(10,i)].y  -- To Last Rank
    )
  end
  
  -- Horizontal Lines
  for i = 1, 10 do
    love.graphics.line(
      gui_board.points[engine.getIndex(i,1)].x, gui_board.points[engine.getIndex(i,1)].y, -- From First File
      gui_board.points[engine.getIndex(i,9)].x, gui_board.points[engine.getIndex(i,9)].y -- To Last File
    )
  end

  -- Fortress Lines
  love.graphics.line(
    gui_board.points[4].x, gui_board.points[4].y,
    gui_board.points[engine.getIndex(3,6)].x, gui_board.points[engine.getIndex(3,6)].y
  )
  love.graphics.line(
    gui_board.points[6].x, gui_board.points[6].y,
    gui_board.points[engine.getIndex(3,4)].x, gui_board.points[engine.getIndex(3,4)].y
  )
  love.graphics.line(
    gui_board.points[engine.getIndex(8,4)].x, gui_board.points[engine.getIndex(8,4)].y,
    gui_board.points[engine.getIndex(10,6)].x, gui_board.points[engine.getIndex(10,6)].y
  )
  love.graphics.line(
    gui_board.points[engine.getIndex(10,4)].x, gui_board.points[engine.getIndex(10,4)].y,
    gui_board.points[engine.getIndex(8,6)].x, gui_board.points[engine.getIndex(8,6)].y
  )

  -- Cannon Points
  love.graphics.circle('fill', gui_board.points[engine.getIndex(3,2)].x, gui_board.points[engine.getIndex(3,2)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(3,8)].x, gui_board.points[engine.getIndex(3,8)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(8,2)].x, gui_board.points[engine.getIndex(8,2)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(8,8)].x, gui_board.points[engine.getIndex(8,8)].y, pradius)

  -- Soldier Points
  love.graphics.circle('fill', gui_board.points[engine.getIndex(4,1)].x, gui_board.points[engine.getIndex(4,1)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(4,3)].x, gui_board.points[engine.getIndex(4,3)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(4,5)].x, gui_board.points[engine.getIndex(4,5)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(4,7)].x, gui_board.points[engine.getIndex(4,7)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(4,9)].x, gui_board.points[engine.getIndex(4,9)].y, pradius)

  love.graphics.circle('fill', gui_board.points[engine.getIndex(7,1)].x, gui_board.points[engine.getIndex(7,1)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(7,3)].x, gui_board.points[engine.getIndex(7,3)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(7,5)].x, gui_board.points[engine.getIndex(7,5)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(7,7)].x, gui_board.points[engine.getIndex(7,7)].y, pradius)
  love.graphics.circle('fill', gui_board.points[engine.getIndex(7,9)].x, gui_board.points[engine.getIndex(7,9)].y, pradius)
  
  -- Notations
  for i, v in pairs(rank_notations) do
    love.graphics.print(i, v.x, v.y)
  end
  for i, v in pairs(file_notations) do
    love.graphics.print(i, v.x, v.y)
  end
end

 -- Flipping the GUI Coordinates only since it shouldn't really affect the real board.
function gui_board.flip()
  -- Board Reference Points
  for i = 1, 45 do -- Half of the Ranks
    local tx = gui_board.points[i].x
    local ty = gui_board.points[i].y

    gui_board.points[i].x = gui_board.points[91-i].x
    gui_board.points[i].y = gui_board.points[91-i].y

    gui_board.points[91-i].x = tx
    gui_board.points[91-i].y = ty
  end
  
  -- Notations
  for i = 1, #rank_notations do
    rank_notations[i].y = gui_board.points[engine.getIndex(i, 1)].y - (font:getHeight(i) / 2)
  end
  for i = 1, #file_notations do
    file_notations[i].x = gui_board.points[i].x - (font:getWidth(1) / 2)
  end
  
  gui_board.flipped = not gui_board.flipped
end
