local image = love.graphics.newImage('img/pieces.png')
local w = image:getWidth()
local h = image:getHeight()
local speed = 500

local quads ={
   ['G'] = love.graphics.newQuad(  0,  0, 60, 60, w, h),
   ['S'] = love.graphics.newQuad( 60,  0, 30, 30, w, h),
   ['A'] = love.graphics.newQuad( 60, 30, 30, 30, w, h),
   ['T'] = love.graphics.newQuad( 90,  0, 40, 40, w, h),
   ['C'] = love.graphics.newQuad(130,  0, 40, 40, w, h),
   ['H'] = love.graphics.newQuad(170,  0, 40, 40, w, h),
   ['E'] = love.graphics.newQuad(210,  0, 40, 40, w, h),

   ['g'] = love.graphics.newQuad(  0, 60, 60, 60, w, h),
   ['s'] = love.graphics.newQuad( 60, 60, 30, 30, w, h),
   ['a'] = love.graphics.newQuad( 60, 90, 30, 30, w, h),
   ['t'] = love.graphics.newQuad( 90, 60, 40, 40, w, h),
   ['c'] = love.graphics.newQuad(130, 60, 40, 40, w, h),
   ['h'] = love.graphics.newQuad(170, 60, 40, 40, w, h),
   ['e'] = love.graphics.newQuad(210, 60, 40, 40, w, h),
}

function newGuiPiece(char, index)
  local piece = {
    char = char,
    index = index,
    x = 0, y = 0,
    vx = 0, vy = 0,
    tx = 0, ty = 0,
    picked_up = false
  }
  _, _, piece.size, _ = quads[char]:getViewport()
  piece.ox = piece.size / 2
  piece.oy = piece.size / 2

  function piece.update(dt)
    if piece.picked_up then
      -- Drag a piece (used on mousepressed)
      piece.x, piece.y = love.mouse.getPosition()
    elseif piece.vx ~= 0 or piece.vy ~= 0 then
      if getCircleCircleDistance(piece.tx, piece.ty, nil, piece.x, piece.y, piece.size/3) < 0 then
        piece.x = piece.tx
        piece.y = piece.ty
        piece.vx = 0
        piece.vy = 0
      else
        piece.x = piece.x + (piece.vx * dt)
        piece.y = piece.y + (piece.vy * dt)
      end
    else
      piece.x = piece.tx
      piece.y = piece.ty
    end
  end
  
  function piece.draw()
    love.graphics.draw(image,quads[piece.char], piece.x, piece.y, 0, 1, 1, piece.ox, piece.oy)
  end
  
  function piece.changeTarget(x, y)
    piece.tx = x
    piece.ty = y
    local angle = getAngle(piece.x, piece.y, x, y)
    piece.vx, piece.vy = getVelocity(speed, angle)
  end
  
  function piece.setPosition(x, y)
    piece.x = x
    piece.y = y
    piece.tx = x
    piece.ty = y
    piece.vx = 0
    piece.vy = 0
  end

  return piece
end

function getGuiPieces(pieces)
  local gui_pieces = {}
  
  for _, v in pairs(pieces) do
    local gui_piece = newGuiPiece(v.char, v.index)
    gui_piece.setPosition(gui_board.points[v.index].x, gui_board.points[v.index].y)
    table.insert(gui_pieces, gui_piece)
  end
  
  return gui_pieces
end
