local library = {
  -- GENERAL
  ['G'] = {value = -1},
  ['S'] = {value =  2},
  ['A'] = {value =  3},
  ['E'] = {value =  3},
  ['H'] = {value =  5},
  ['C'] = {value =  7},
  ['T'] = {value = 13}
}

local function canMoveTo(points, index1, index2, check_legality)
  local can_move = (points[index2] == ' ' or engine.getColor(points[index1]) ~= engine.getColor(points[index2]))
  if can_move and check_legality then
    local b = newBoard(engine.getRepresentation(points), engine.getColor(points[index1]))
    b.movePiece(index1, index2)
    
    if engine.isBikjang(points) and engine.isBikjang(b.points) then
      can_move = false
    elseif engine.isJanggun(b.points, engine.getColor(points[index1])) then
      can_move = false
    end
  end
  return can_move
end

--ADVISER MOVES
library['A'].getMoves = function(points, index, rank, file, char, check_legality)
  local moves = {}
  --Files
  if file < 6 and canMoveTo(points, index, index+1, check_legality) then
     table.insert(moves, index+1) -- Index+1 = File to the Right
  end
  if file > 4 and canMoveTo(points, index, index-1, check_legality) then
     table.insert(moves, index-1) -- Index-1 = File to the Left
  end
  --Ranks
  if (rank == 1 or rank == 2 or rank == 8 or rank == 9) then
    if canMoveTo(points, index, engine.getIndex(rank+1, file), check_legality) then
      table.insert(moves, engine.getIndex(rank+1, file))
    end
  end
  if (rank == 2 or rank == 3 or rank == 9 or rank == 10) then
    if canMoveTo(points, index, engine.getIndex(rank-1, file), check_legality) then
      table.insert(moves, engine.getIndex(rank-1, file))
    end
  end
  --Fortress Points
  if (index == engine.getIndex(1,4) or index == engine.getIndex(2,5) or
      index == engine.getIndex(8,4) or index == engine.getIndex(9,5)) and
     canMoveTo(points, index, engine.getIndex(rank+1, file+1), check_legality) then
     table.insert(moves, engine.getIndex(rank+1, file+1))
  end
  if (index == engine.getIndex(1,6) or index == engine.getIndex(2,5) or
      index == engine.getIndex(8,6) or index == engine.getIndex(9,5)) and
     canMoveTo(points, index, engine.getIndex(rank+1, file-1), check_legality) then
     table.insert(moves, engine.getIndex(rank+1, file-1))
  end
  if (index == engine.getIndex( 3,4) or index == engine.getIndex(2,5) or
      index == engine.getIndex(10,4) or index == engine.getIndex(9,5)) and
     canMoveTo(points, index, engine.getIndex(rank-1, file+1), check_legality) then
     table.insert(moves, engine.getIndex(rank-1, file+1))
  end
  if (index == engine.getIndex( 3,6) or index == engine.getIndex(2,5) or
      index == engine.getIndex(10,6) or index == engine.getIndex(9,5)) and
     canMoveTo(points, index, engine.getIndex(rank-1, file-1), check_legality) then
     table.insert(moves, engine.getIndex(rank-1, file-1))
  end

  return moves
end

--GENERALS MOVES: Same as ADVISER, with an additional rule for bikjang
library['G'].getMoves = function(points, index, rank, file, char, check_legality)
  local moves = library['A'].getMoves(points, index, rank, file, char, check_legality)
  return moves
end

--SOLDIER MOVES:
library['S'].getMoves = function(points, index, rank, file, char, check_legality)
  local moves = {}
  --Both Soldiers
  if file > 1 and canMoveTo(points,index,index-1, check_legality) then
    table.insert(moves,index-1)
  end
  if file < 9 and canMoveTo(points,index,index+1, check_legality) then
    table.insert(moves,index+1)
  end
  --Chu Moving Forward
  if engine.getColor(char) == 1 then
    if rank > 1 and canMoveTo(points,index,index-9, check_legality) then --9 = 1 Rank
      table.insert(moves,index-9)
    end
    --Fortress
    if (index == engine.getIndex(3,4) or index == engine.getIndex(3,6)) and
       canMoveTo(points,index,engine.getIndex(2,5), check_legality) then
       table.insert(moves,engine.getIndex(2,5))
    end
    if index == engine.getIndex(2,5) then
      if canMoveTo(points,index,index-10, check_legality) then
        table.insert(moves,index-10)
      end
      if canMoveTo(points,index,index-8, check_legality) then
        table.insert(moves,index-8)
      end
    end

  --Han Moving Forward
  else
    if rank < 10 and canMoveTo(points,index,index+9, check_legality) then
      table.insert(moves,index+9)
    end
    --Fortress
    if (index == engine.getIndex(8,4) or index == engine.getIndex(8,6)) and
       canMoveTo(points,index,engine.getIndex(9,5), check_legality) then
       table.insert(moves,engine.getIndex(9,5))
    end
    if index == engine.getIndex(9,5) then
      if canMoveTo(points,index,index+8, check_legality) then
        table.insert(moves,index+8)
      end
      if canMoveTo(points,index,index+10, check_legality) then
        table.insert(moves,index+10)
      end
    end
  end
  
  return moves
end

--ELEPHANT MOVES
library['E'].getMoves = function(points, index, rank, file, char, check_legality)
  local moves = {}
  --Moving Up
  if rank > 3 and points[index-9] == ' ' then
    if file > 2 and points[engine.getIndex(rank-2, file-1)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank-3,file-2), check_legality) then
        table.insert(moves, engine.getIndex(rank-3,file-2))
      end
    end
    if file < 8 and points[engine.getIndex(rank-2,file+1)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank-3,file+2), check_legality) then
        table.insert(moves, engine.getIndex(rank-3,file+2))
      end
    end
  end
  --Moving Down
  if rank < 8 and points[index+9] == ' ' then
    if file > 2 and points[engine.getIndex(rank+2,file-1)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank+3,file-2), check_legality) then
        table.insert(moves, engine.getIndex(rank+3,file-2))
      end
    end
    if file < 8 and points[engine.getIndex(rank+2,file+1)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank+3,file+2), check_legality) then
        table.insert(moves, engine.getIndex(rank+3,file+2))
      end
    end
  end
  --Moving Left
  if file > 3 and points[index-1] == ' ' then
    if rank > 2 and points[engine.getIndex(rank-1,file-2)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank-2,file-3), check_legality) then
         table.insert(moves, engine.getIndex(rank-2,file-3))
      end
    end
    if rank < 9 and points[engine.getIndex(rank+1,file-2)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank+2,file-3), check_legality) then
        table.insert(moves,engine.getIndex(rank+2,file-3))
      end
    end
  end
  --Moving Right
  if file < 7 and points[index+1] == ' ' then
    if rank > 2 and points[engine.getIndex(rank-1,file+2)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank-2,file+3), check_legality) then
        table.insert(moves,engine.getIndex(rank-2,file+3))
      end
    end
    if rank < 9 and points[engine.getIndex(rank+1,file+2)] == ' ' then
      if canMoveTo(points,index,engine.getIndex(rank+2,file+3), check_legality) then
        table.insert(moves,engine.getIndex(rank+2,file+3))
      end
    end
  end
  
  return moves
end

--HORSE MOVES
library['H'].getMoves = function(points, index, rank, file, char, check_legality)
  local moves = {}

  --Moving Up
  if rank > 2 and points[index-9] == ' ' then
    if file > 1 and canMoveTo(points,index,engine.getIndex(rank-2,file-1), check_legality) then
      table.insert(moves,engine.getIndex(rank-2,file-1))
    end
    if file < 9 and canMoveTo(points,index,engine.getIndex(rank-2,file+1), check_legality) then
      table.insert(moves,engine.getIndex(rank-2,file+1))
    end
  end
  --Moving Down
  if rank < 9 and points[index+9] == ' ' then
    if file > 1 and canMoveTo(points,index,engine.getIndex(rank+2,file-1), check_legality) then
      table.insert(moves,engine.getIndex(rank+2,file-1))
    end
    if file < 9 and canMoveTo(points,index,engine.getIndex(rank+2,file+1), check_legality) then
      table.insert(moves,engine.getIndex(rank+2,file+1))
    end
  end
  --Moving Left
  if file > 2 and points[index-1] == ' ' then
    if rank > 1 and canMoveTo(points,index,engine.getIndex(rank-1,file-2), check_legality) then
      table.insert(moves,engine.getIndex(rank-1,file-2))
    end
    if rank < 10 and canMoveTo(points,index,engine.getIndex(rank+1,file-2), check_legality) then
      table.insert(moves,engine.getIndex(rank+1,file-2))
    end
  end
  --Moving Right
  if file < 8 and points[index+1] == ' ' then
    if rank > 1 and canMoveTo(points,index,engine.getIndex(rank-1,file+2), check_legality) then
      table.insert(moves,engine.getIndex(rank-1,file+2))
    end
    if rank < 10 and canMoveTo(points,index,engine.getIndex(rank+1,file+2), check_legality) then
      table.insert(moves,engine.getIndex(rank+1,file+2))
    end
  end

  return moves
end

--CANNON MOVES
library['C'].getMoves = function(points, index, rank, file, char, check_legality)
  local moves = {}

  local iterator
  local jumping_piece
  --Moving Up
  if rank > 2 then
    iterator = rank
    jumping_piece = false -- First Find a jumping piece
    while iterator > 1 and jumping_piece == false do
      iterator = iterator - 1
      if points[engine.getIndex(iterator,file)]:upper() == 'C' then
        iterator = 1
      elseif points[engine.getIndex(iterator,file)] ~= ' ' then
        jumping_piece = true
      end
    end

    if jumping_piece then
      while iterator > 1 do
        iterator = iterator - 1
        local index2 = engine.getIndex(iterator, file)
        if points[index2]:upper() == 'C' then
          iterator = 1
        elseif canMoveTo(points,index, index2, check_legality) then
          table.insert(moves, index2)
        end
        if points[index2] ~= ' ' then
          iterator = 1
        end
      end
    end
  end
  --Moving Down
  if rank < 9 then
    iterator = rank
    jumping_piece = false
    while iterator < 10 and jumping_piece == false do
      iterator = iterator + 1
      if points[engine.getIndex(iterator,file)]:upper() == 'C' then
        iterator = 10
      elseif points[engine.getIndex(iterator,file)] ~= ' ' then
        jumping_piece = true
      end
    end

    if jumping_piece then
      while iterator < 10 do
        iterator = iterator + 1
        local index2 = engine.getIndex(iterator, file)
        if points[index2]:upper() == 'C' then
          iterator = 10
        elseif canMoveTo(points,index, index2, check_legality) then
          table.insert(moves, index2)
        end
        if points[index2] ~= ' ' then
          iterator = 10
        end
      end
    end
  end
  --Moving Left
  if file > 2 then
    iterator = file
    jumping_piece = false
    while iterator > 1 and jumping_piece == false do
      iterator = iterator - 1
      if points[engine.getIndex(rank,iterator)]:upper() == 'C' then
        iterator = 1
      elseif points[engine.getIndex(rank,iterator)] ~= ' ' then
        jumping_piece = true
      end
    end
    
    if jumping_piece then
      while iterator > 1 do
        iterator = iterator - 1
        local index2 = engine.getIndex(rank, iterator)
        if points[index2]:upper() == 'C' then
          iterator = 1
        elseif canMoveTo(points, index, index2, check_legality) then
          table.insert(moves, index2)
        end
        if points[index2] ~= ' ' then
            iterator = 1
        end
      end
    end
  end
  --Moving Right
  if file < 8 then
    iterator = file
    jumping_piece = false
    while iterator < 9 and jumping_piece == false do
      iterator = iterator + 1
      if points[engine.getIndex(rank,iterator)]:upper() == 'C' then
        iterator = 9
      elseif points[engine.getIndex(rank,iterator)] ~= ' ' then
        jumping_piece = true
      end
    end
    
    if jumping_piece then
      while iterator < 9 do
        iterator = iterator + 1
        local index2 = engine.getIndex(rank, iterator)
        if points[index2]:upper() == 'C' then
          iterator = 9
        elseif canMoveTo(points, index, index2, check_legality) then
          table.insert(moves, index2)
        end
        if points[index2] ~= ' ' then
          iterator = 9
        end
      end
    end
  end
  --Fortress Points
  if index == engine.getIndex(1,4) or index == engine.getIndex(8,4) then
    if points[engine.getIndex(rank+1,file+1)] ~= ' ' and points[engine.getIndex(rank+1,file+1)]:upper() ~= 'C' and
       points[engine.getIndex(rank+2,file+2)]:upper() ~= 'C' and canMoveTo(points,index,engine.getIndex(rank+2,file+2), check_legality) then
       table.insert(moves,engine.getIndex(rank+2,file+2))
    end
  end
  if index == engine.getIndex(1,6) or index == engine.getIndex(8,6) then
    if points[engine.getIndex(rank+1,file-1)] ~= ' ' and points[engine.getIndex(rank+1,file-1)]:upper() ~= 'C' and
       points[engine.getIndex(rank+2,file-2)]:upper() ~= 'C' and canMoveTo(points,index,engine.getIndex(rank+2,file-2), check_legality) then
       table.insert(moves,engine.getIndex(rank+2,file-2))
    end
  end
  if index == engine.getIndex(3,4) or index == engine.getIndex(10,4) then
    if points[engine.getIndex(rank-1,file+1)] ~= ' ' and points[engine.getIndex(rank-1,file+1)]:upper() ~= 'C' and
       points[engine.getIndex(rank-2,file+2)]:upper() ~= 'C' and canMoveTo(points,index,engine.getIndex(rank-2,file+2), check_legality) then
       table.insert(moves,engine.getIndex(rank-2,file+2))
    end
  end
  if index == engine.getIndex(3,6) or index == engine.getIndex(10,6) then
    if points[engine.getIndex(rank-1,file-1)] ~= ' ' and points[engine.getIndex(rank-1,file-1)]:upper() ~= 'C' and
       points[engine.getIndex(rank-2,file-2)]:upper() ~= 'C' and canMoveTo(points,index,engine.getIndex(rank-2,file-2), check_legality) then
       table.insert(moves,engine.getIndex(rank-2,file-2))
    end
  end

  return moves
end

--CHARIOT MOVES
library['T'].getMoves = function(points, index, rank, file, char, check_legality)
  local moves = {}

  local iterator
  --Moving Up
  if rank > 1 then
    iterator = rank
    while iterator > 1 do
      iterator = iterator - 1
      local index2 = engine.getIndex(iterator,file)

      if canMoveTo(points, index, index2, check_legality) then
        table.insert(moves, index2)
      end
      if points[index2] ~= ' ' then -- Blocking Piece Found
        iterator = 0
      end
    end
  end
  --Moving Down
  if rank < 10 then
    iterator = rank
    while iterator < 10 do
      iterator = iterator + 1
      local index2 = engine.getIndex(iterator,file)

      if canMoveTo(points, index, index2, check_legality) then
        table.insert(moves, index2)
      end
      if points[index2] ~= ' ' then -- Blocking Piece Found
        iterator = 11
      end
    end
  end
  --Moving Left
  if file > 1 then
    iterator = file
    while iterator > 1 do
      iterator = iterator - 1
      local index2 = engine.getIndex(rank, iterator)

      if canMoveTo(points, index, index2, check_legality) then
        table.insert(moves, index2)
      end
      if points[index2] ~= ' ' then -- Blocking Piece Found
        iterator = 0
      end
    end
  end
  --Moving Right
  if file < 9 then
    iterator = file
    while iterator < 9 do
      iterator = iterator + 1
      local index2 = engine.getIndex(rank, iterator)

      if canMoveTo(points, index, index2, check_legality) then
        table.insert(moves, index2)
      end
      if points[index2] ~= ' ' then -- Blocking Piece Found
        iterator = 10
      end
    end
  end
  --Fortress Points
  if index == engine.getIndex(1,4) or index == engine.getIndex(8,4) then
    if canMoveTo(points,index,engine.getIndex(rank+1,file+1), check_legality) then
      table.insert(moves, engine.getIndex(rank+1, file+1))
    end
    if points[engine.getIndex(rank+1,file+1)] == ' ' and
       canMoveTo(points,index,engine.getIndex(rank+2,file+2), check_legality) then
       table.insert(moves, engine.getIndex(rank+2,file+2))
    end
  end
  if index == engine.getIndex(1,6) or index == engine.getIndex(8,6) then
    if canMoveTo(points,index,engine.getIndex(rank+1,file-1), check_legality) then
      table.insert(moves, engine.getIndex(rank+1, file-1))
    end
    if points[engine.getIndex(rank+1,file-1)] == ' ' and
       canMoveTo(points,index,engine.getIndex(rank+2,file-2), check_legality) then
       table.insert(moves, engine.getIndex(rank+2,file-2))
    end
  end
  if index == engine.getIndex(3,4) or index == engine.getIndex(10,4) then
    if canMoveTo(points,index,engine.getIndex(rank-1,file+1), check_legality) then
      table.insert(moves, engine.getIndex(rank-1, file+1))
    end
    if points[engine.getIndex(rank-1,file+1)] == ' ' and
       canMoveTo(points,index,engine.getIndex(rank-2,file+2), check_legality) then
       table.insert(moves, engine.getIndex(rank-2,file+2))
    end
  end
  if index == engine.getIndex(3,6) or index == engine.getIndex(10,6) then
    if canMoveTo(points,index,engine.getIndex(rank-1,file-1), check_legality) then
      table.insert(moves, engine.getIndex(rank-1, file-1))
    end
    if points[engine.getIndex(rank-1,file-1)] == ' ' and
       canMoveTo(points,index,engine.getIndex(rank-2,file-2), check_legality) then
       table.insert(moves, engine.getIndex(rank-2,file-2))
    end
  end
  if index == engine.getIndex(2,5) or index == engine.getIndex(9,5) then
    if canMoveTo(points,index,engine.getIndex(rank-1,file-1), check_legality) then
      table.insert(moves,engine.getIndex(rank-1,file-1))
    end
    if canMoveTo(points,index,engine.getIndex(rank-1,file+1), check_legality) then
      table.insert(moves,engine.getIndex(rank-1,file+1))
    end
    if canMoveTo(points,index,engine.getIndex(rank+1,file-1), check_legality) then
      table.insert(moves,engine.getIndex(rank+1,file-1))
    end
    if canMoveTo(points,index,engine.getIndex(rank+1,file+1), check_legality) then
      table.insert(moves,engine.getIndex(rank+1,file+1))
    end
  end

  return moves
end

-- Non-GUI Piece
function newPiece(index, char)
  local piece = {
    index = index,
    char = char,
    value = library[char:upper()].value,
    moves = {}
  }
    
  function piece.getMoves(points, check_legality)
    piece.moves = library[piece.char:upper()].getMoves(
      points,
      piece.index,
      engine.getRank(piece.index),
      engine.getFile(piece.index),
      piece.char,
      check_legality
    )
  end
    
  return piece
end
