local width = 188
local height = 164
local font = love.graphics.newFont('font/Fontin-Bold.otf', 14)
local font_maintimer = love.graphics.newFont('font/Bergen Mono Regular.otf', 35)
local font_smalltimer = love.graphics.newFont('font/Bergen Mono Regular.otf', 20)

function newPlayer(x, y, vars)
  local player = {
    x = x or 0,
    y = y or 0,
    backcolor = vars.backcolor or {0, 0, 0},
    color = vars.color or 1,
    name = vars.name or 'Anonymous',
    score = 0,
    timerstatus = vars.timerstatus or 'off',
    text = ''
  }
  
  player.menu = newMenu(nil, 4)
  player.menu.addButton('Skip Turn', x + (width / 4), y + 65)
  player.menu.addButton('Resign', x + ((width / 4) *3), y + 65)
  
  if player.timerstatus == 'on' then
    local data = {}
    for line in love.filesystem.lines('timer.cfg') do
      table.insert(data, line)
    end
    local main = tonumber(data[2])
    local byoyomi = tonumber(data[3])
    local periods = tonumber(data[4])
    player.timer = newTimer(main, byoyomi, periods)
  end
  
  function player.update(dt)
    if player.timerstatus == 'on' then
      player.timer.update(dt)
    end
  end
  
  function player.computeScore(pieces)
    player.score = player.color == 2 and 1.5 or 0
    for _, v in pairs(pieces) do
      if v.char:upper() ~= 'G' and engine.getColor(v.char) == player.color then
        player.score = player.score + v.value
      end
    end
  end
  
  function player.draw(color)
    local name_height = font:getHeight(player.name) * 3
    if color == player.color then
      love.graphics.setColor(player.backcolor)
      love.graphics.rectangle('fill', player.x, player.y, width, name_height)
      player.menu.draw()
    end
    love.graphics.setColor(1,1,1)
    love.graphics.rectangle('line', player.x, player.y, width, height)
    love.graphics.line(player.x, player.y + name_height, player.x + width, player.y + name_height)
    love.graphics.setFont(font)
    love.graphics.printf(player.name, player.x, player.y, width, 'center')
    love.graphics.printf(player.score, player.x, player.y + 25, width, 'center')
    love.graphics.line(player.x, player.y + 85, player.x + width, player.y + 85)
    love.graphics.setFont(font_maintimer)
    local text = player.timerstatus == 'off' and '--:--:--' or player.timer.main_text()
    love.graphics.printf(text, player.x, player.y + 85, width, 'center')
    love.graphics.line(player.x, player.y + 130, player.x + width, player.y + 130)
    love.graphics.setFont(font_smalltimer)
    text = player.timerstatus == 'off' and '--' or player.timer.periods_text()
    love.graphics.printf(text, player.x, player.y + 135, 40, 'center')
    text = player.timerstatus == 'off' and '--:--' or player.timer.byoyomi_text()
    love.graphics.printf(text, player.x + 40, player.y + 135, width - 40, 'center')
    love.graphics.line(player.x + 40, player.y + 130, player.x + 40, player.y + 163)
  end
  
  return player
end
