state_analyzematch = newState()

local board = {}
local gui_pieces = {}
local max_radius = 13
local turn_counter = 0
local players = {}

local menu = newGMenu()
local controls = newGMenu()
local setmover = newGMenu()

local sidebar = {
  x = 603, y = 65,
  width = 188, height = 490
}

local state_analyzer = {
  cols = 2,
  colwidth = 100,
  rowheight = 20,
  set_size = 48,
  current_set = 0,
  move_history = {}
}

local gui_move_history = {}

local function flipBoard()
  gui_board.flip()
  for _, v in pairs(gui_pieces) do
    v.setPosition(gui_board.points[v.index].x, gui_board.points[v.index].y)
  end
end

local function guiRedoMove()
  if board.history_position < #board.move_history then
    -- GUI: Now we can do this
    local from = board.move_history[board.history_position+1].from
    local to = board.move_history[board.history_position+1].to
    
    if from ~= to then
      -- Step 1 - Remove the piece that has the same index as "to"
      for i, v in pairs(gui_pieces) do
        if v.index == to then table.remove(gui_pieces, i) end
      end
    
      -- Step 2 - Re-assign the piece that has the same index as "from"
      for _, v in pairs(gui_pieces) do
        if v.index == from then
          v.index = to
          v.changeTarget(gui_board.points[to].x, gui_board.points[to].y)
        end
      end
    end
    turn_counter = (turn_counter%2) + 1
  end

  -- Internal Function
  board.redoMove()
  state_analyzer.current_set = math.ceil(board.history_position/state_analyzer.set_size)
end

function guiUndoMove()
  -- GUI Version First
  if board.history_position > 0 then
    local from = board.move_history[board.history_position].to
    local to = board.move_history[board.history_position].from
    local char = board.move_history[board.history_position].char
    
    if from ~= to then
      -- Re-assigning the piece that has the same index as "from"
      for _, v in pairs(gui_pieces) do
        if v.index == from then
          v.index = to
          v.changeTarget(gui_board.points[to].x, gui_board.points[to].y)
        end
      end
    
      -- Adding a piece back in if it was captured
      if char ~= ' ' then
        local gui_piece = newGuiPiece(char, from)
        gui_piece.setPosition(gui_board.points[from].x, gui_board.points[from].y)
        table.insert(gui_pieces, gui_piece)
      end
    end

    turn_counter = (turn_counter%2) + 1
  end
    
  -- Internal Version
  board.undoMove()
  state_analyzer.current_set = math.ceil(board.history_position/state_analyzer.set_size)
end

local function guiGotoMove(i)
  if board.history_position < i then
    repeat
      guiRedoMove()
    until board.history_position == i
  elseif board.history_position > i then
    repeat
      guiUndoMove()
    until board.history_position == i
  end
  board.checkMoves(turn_counter, true)
end

local function displaySet()
  gui_move_history = {}
  state_analyzer.current_set = state_analyzer.current_set == 0 and 1 or state_analyzer.current_set
  for i = 1, #state_analyzer.move_history do
    if math.ceil(i/state_analyzer.set_size) == state_analyzer.current_set then
      table.insert(gui_move_history, {text = state_analyzer.move_history[i], index = i})
    end
  end
  
  for i, v in pairs(gui_move_history) do
    local row = math.ceil(i/state_analyzer.cols)
    local col = ((i-1)%state_analyzer.cols)+1
    if col == 1 then
      v.text = math.ceil(v.index/state_analyzer.cols) .. '. ' .. v.text
    end
    v.x = sidebar.x + 15 + ((col-1) * state_analyzer.colwidth)
    v.y = sidebar.y + 5 + ((row-1) * state_analyzer.rowheight)
  end
end

menu.addButton('Play from Position', 69, 15, function()
  if board.history_position < #board.move_history then
    state_machine.change(state_play, {eval = true, turn_counter = turn_counter, rep = engine.getRepresentation(board.points)})
  end
end)
menu.addButton('Flip Board',    404, 15, function() flipBoard()                       end)
menu.addButton('Exit Analysis', 532, 15, function() state_machine.change(state_title) end)

controls.addButton('<<',187, 15, function()
  while board.history_position > 0 do
    guiUndoMove()
  end
  board.checkMoves(turn_counter, true)
  displaySet()
end)
controls.addButton('<', 218, 15, function() guiUndoMove() board.checkMoves(turn_counter, true) displaySet() end)
controls.addButton('>', 249, 15, function() guiRedoMove() board.checkMoves(turn_counter, true) displaySet() end)
controls.addButton('>>',280, 15, function()
  while board.history_position < #board.move_history do
    guiRedoMove()
  end
  board.checkMoves(turn_counter, true)
  displaySet()
end)

setmover.addButton('[', 665, 570, function()
  state_analyzer.current_set = 1
  displaySet()
end)
setmover.addButton('-',  684, 570, function()
  state_analyzer.current_set = state_analyzer.current_set == 1 and 1 or state_analyzer.current_set - 1
  displaySet()
end)
setmover.addButton('+',  703, 570, function()
  state_analyzer.current_set = state_analyzer.current_set == math.ceil(#board.move_history/state_analyzer.set_size) and state_analyzer.current_set or state_analyzer.current_set + 1
  displaySet()
end)
setmover.addButton(']', 722, 570, function()
  state_analyzer.current_set = math.ceil(#board.move_history/state_analyzer.set_size)
  displaySet()
end)

function state_analyzematch.load(params)
  gui_board.load()

  turn_counter = params.turn_counter
  local rep = params.rep
  board = newBoard(rep, turn_counter, true, params.move_history)
  
  gui_pieces = getGuiPieces(board.pieces)

  state_analyzer.move_history = {}
  local history_backwards = {}
  while board.history_position > 0 do
    local index1 = board.move_history[board.history_position].from
    local index2 = board.move_history[board.history_position].to
    local char = board.points[index2]
    local text = engine.getRank(index1) .. engine.getFile(index1) .. char:upper() ..
                 engine.getRank(index2) .. engine.getFile(index2)
    table.insert(history_backwards, text)
    board.move_history[board.history_position].rep = engine.getRepresentation(board.points)
    guiUndoMove()
  end
  board.checkMoves(turn_counter, true)
  
  for i = #history_backwards, 1, -1 do
    table.insert(state_analyzer.move_history, history_backwards[i])
  end
  displaySet()
  
  players = {}
  table.insert(players, {name = params.player1})
  table.insert(players, {name = params.player2})
end

function state_analyzematch.update(dt)
  for _, v in pairs(gui_pieces) do
    v.update(dt)
  end
end

function state_analyzematch.draw()
  local static = {}
  local moving = {}
  for i, v in pairs(gui_pieces) do
    if v.vx ~= 0 or v.vy ~= 0 then
      table.insert(moving, i)
    else
      table.insert(static, i)
    end
  end

  --Order 1: Non-Moving Pieces
  for _, v in pairs(static) do
    gui_pieces[v].draw()
  end
  --Order 2: Moving Pieces
  love.graphics.setColor(1,1,1)
  for _, v in pairs(moving) do
    gui_pieces[v].draw()
  end
  --Order 3: (GUI-from-Internal) Previous Move Indicator
  if board.history_position > 0 then
    -- GUI: Indicate the color that moved previously
    local prev_color = turn_counter == 2 and {0, 120/255, 0, 0.9} or {175/255, 0, 0, 0.7}
    love.graphics.setColor(prev_color)
    
    -- Internal: get the index of the move history from the position
    local index = board.move_history[board.history_position].from    
    love.graphics.rectangle('line',
      gui_board.points[index].x - max_radius,
      gui_board.points[index].y - max_radius,
      max_radius * 2, max_radius * 2,
      3, 3
    )
  end

  love.graphics.setColor(1,1,1)
  -- Sidebar Outline
  love.graphics.rectangle('line', sidebar.x, sidebar.y, sidebar.width, sidebar.height)
  
  -- Evaluation
  love.graphics.printf(board.hard_evaluation, sidebar.x, 40, sidebar.width, 'center')
  
  for i, v in pairs(gui_move_history) do
    -- History Moves
    love.graphics.print(v.text, v.x, v.y)

    -- History Position Indicator
    if v.index == board.history_position then
      love.graphics.setColor(1,1,1, 0.4)
      love.graphics.rectangle('fill', v.x - 10, v.y, 75, state_analyzer.rowheight)
      love.graphics.setColor(1,1,1)
    end
    
    -- Mouse Pointer Indicator
    if getCircleRectangleDistance(love.mouse.getX(), love.mouse.getY(), 1, v.x - 10, v.y, 75, state_analyzer.rowheight) < 0 then
      love.graphics.rectangle('line', v.x - 10, v.y, 75, state_analyzer.rowheight)
    end
  end

  --Menu
  menu.draw()
  controls.draw()
  setmover.draw()
end

function state_analyzematch.keypressed(key)
  if key == 'space' then
    flipBoard()
  end
end

function state_analyzematch.mousemoved(x, y, dx, dy)
  menu.mouse_over(x, y)
  controls.mouse_over(x, y)
  setmover.mouse_over(x, y)
end
function state_analyzematch.mousepressed(x, y, button)
  if button == 1 then
    menu.button_press()
    controls.button_press()
    setmover.button_press()
    for _, v in pairs(gui_move_history) do
      if getCircleRectangleDistance(x, y, 1, v.x - 10, v.y, 75, state_analyzer.rowheight) < 0 then
        guiGotoMove(v.index)
      end
    end
  end
end
function state_analyzematch.mousereleased(x, y, button)
  if button == 1 then
    menu.button_release()
    controls.button_release()
    setmover.button_release()
  end
end
