state_loadmatch = newState()

local bx = 10
local by = 35
local bw = 580
local bh = 550
local list = {}

local row_height = 20
local cols = 5
local col_width = 115
local selector = { x = 0, y = 0, index = 0}

local function loadList()
  local items = love.filesystem.getDirectoryItems('')
  list = {}
  for _, v in pairs(items) do
    if v:sub(-4) == '.mch' then
      local i = #list+1
      local row = math.ceil(i/cols)
      local col = ((i-1)%cols)+1
      table.insert(list, {filename = v:sub(1,#v-4), x = bx + 5 + (col-1) * col_width, y = by + 5 + (row-1) * row_height})
    end
  end
  
  selector.index = 0
end

local deleter = newGSubMenu()
deleter.addButton('x', 0, 0, function()
    love.filesystem.remove(list[selector.index].filename .. '.mch')
    loadList()
  end
)

function state_loadmatch.load()
  loadList()
end

function state_loadmatch.draw()
  love.graphics.setColor(0,0,0,0.4)
  love.graphics.rectangle('fill', bx, by, bw, bh, 4, 4)
  
  love.graphics.setColor(1,1,1)
  for i, v in pairs(list) do
    love.graphics.print(v.filename, v.x, v.y)
  end
  
  if selector.index > 0 then
    love.graphics.setColor(1,1,1,0.3)
    love.graphics.rectangle('fill', selector.x, selector.y, col_width-12, row_height)
    deleter.draw()
  end
end

function state_loadmatch.mousemoved(x, y, dx, dy)
  selector.index = 0
  for i, v in pairs(list) do
    if getCircleRectangleDistance(x, y, 1, v.x, v.y, col_width, row_height) < 0 then
      selector.x = v.x
      selector.y = v.y
      selector.index = i
      deleter.buttons[1].x = selector.x + col_width - 6
      deleter.buttons[1].y = selector.y + 10
    end
  end
  if selector.index > 0 then
    deleter.mouse_over(x, y)
  end
end
function state_loadmatch.mousepressed(x, y, button)
  if button == 2 or x < bx or y < by or x > bx + bw or y > by + bh then
    state_machine.pop()
  elseif button == 1 and selector.index > 0 then
    deleter.button_press()
  end
end
function state_loadmatch.mousereleased(x, y, button)
  if button == 1 and selector.index > 0 then
    deleter.button_release()
    if selector.index > 0 then
      -- Load Game
      local data = {}
      for line in love.filesystem.lines(list[selector.index].filename .. '.mch') do
        table.insert(data, line)
      end
      local player1 = data[1]
      local player2 = data[2]
      local rep = data[3]
      local turn_counter = tonumber(data[4])
      local move_history = {}
      for i = 5, #data do
        local char = data[i]:sub(1,1)
        local from = tonumber(data[i]:sub(2,3))
        local to = tonumber(data[i]:sub(4,5))
        table.insert(move_history, {char = char, from = from, to = to})
      end
      
      state_machine.change(state_analyzematch, {player1 = player1, player2 = player2, rep = rep, turn_counter = turn_counter, move_history = move_history})
    end
  end
end
