state_play = newState()

local board = {}
local menu = {}
local gui_pieces = {}
local gui_moves = {}
local selected_index = 0
local max_radius = 13
local cur_radius = 0
local spd_radius = 100
local turn_counter = 0
local players = {}
local ai
local font = love.graphics.newFont('font/YanoneKaffeesatz-Bold.ttf', 20)
local who_won = -1
local who_really_won = -1
local text = ''
local timerstatus = 'off'

local function declareSituation()
  who_won, who_really_won = engine.whoWon(board.hard_evaluation, board.pieces, turn_counter)
  text = board.hard_evaluation
  
  if who_won ~= -1 then
    table.remove(menu.buttons, 1)
    menu.addButton('Save Match',52, 15, function() state_machine.push(state_savematch, {
        player1 = players[1].name,
        player2 = players[2].name,
        representation = engine.getRepresentation(board.points),
        turn_counter = turn_counter,
        move_history = board.move_history
        })
      end
    )
    menu.addButton('Play Again', 337, 15, function() state_machine.change(state_hansetup, ai) end)
  end
  
  if board.hard_evaluation == 'Exposed General' then
    text = text .. '\n' .. players[turn_counter].name .. "'s General is Under Attack"
  elseif board.hard_evaluation == 'Truce Offered' then
    local other_player = turn_counter == 1 and 2 or 1
    text = text .. '\n' .. players[turn_counter].name .. ' is being invited to ' .. players[other_player].name .. "'s Fortress"
  elseif who_won == 0 then
    text = text .. '\n(Positional Draw)' .. '\n' ..
    players[who_really_won].name .. ' Wins by Piece Points'
  else
    local loser = who_won == 1 and 2 or 1
    if board.hard_evaluation == 'Outside General' then
      text = text .. '\n' .. players[loser].name .. ' surrenders by stepping outside the Fortress'
    elseif board.hard_evaluation == 'Slain General' then
      text = text .. '\n' .. players[loser].name .. " can't escape death"
    end
  end
end

local function flipBoard()
  gui_board.flip()
  for _, v in pairs(gui_pieces) do
    v.setPosition(gui_board.points[v.index].x, gui_board.points[v.index].y)
  end

  players[1].y = gui_board.flipped and 65 or 395
  players[2].y = gui_board.flipped and 395 or 65
    
  for _, v in pairs(players) do
    for _, b in pairs(v.menu.buttons) do
      b.y = v.y + 65
    end
  end
end

local function guiSelectMove(x, y, another_radius)
  local selected_move = 0
  local radius = another_radius or nil
  for _, v in pairs(gui_moves) do
    if getCircleCircleDistance(x, y, radius, gui_board.points[v].x, gui_board.points[v].y, max_radius) < 0 then
      selected_move = v
    end
  end
  return selected_move
end

local function guiSelectPiece(x, y)
  -- Internal: Detects if a piece has been picked up
  local picked_up_index = 0
  gui_moves = {} -- Empty GUI Moves
  
  for i, v in pairs(gui_pieces) do
    if getCircleCircleDistance(x, y, nil, v.x, v.y, v.size/2) < 0 then
      v.picked_up = true        --GUI: Pick it up
      picked_up_index = v.index --Internal: Get the index
    end
  end
    
  --Internal: Look for the internal counterpart of the picked up piece
  if picked_up_index > 0 then
    local internal_index = 0
    for i, v in pairs(board.pieces) do
      -- If found, get the true index of the piece
      if v.index == picked_up_index then
        internal_index = i
      end
    end

    -- Internal-to-GUI: Now, give GUI Moves from the internal piece...
    for _, v in pairs(board.pieces[internal_index].moves) do
      table.insert(gui_moves, v)
    end
  end
  
  return picked_up_index
end

local function guiMovePiece(from, to, redo)
  -- Internal: Do the internal counterpart first
  board.movePiece(from, to, redo)
  
  -- GUI: Now do the GUI version of the function above
  if from ~= to then
    -- Step 1 - Remove the piece that has the same index as "to"
    for i, v in pairs(gui_pieces) do
      if v.index == to then
        table.remove(gui_pieces, i)
      end
    end
    
    -- Step 2 - Re-assign the piece that has the same index as "from"
    for _, v in pairs(gui_pieces) do
      if v.index == from then
        v.index = to
        v.changeTarget(gui_board.points[to].x, gui_board.points[to].y)
      end
    end
  end
    
  -- GAME:
  gui_moves = {} -- Empty GUI Moves
  turn_counter = (turn_counter%2) + 1 -- Next Player's Turn
  board.checkMoves(turn_counter, true) -- Internal: Re-evaluate Piece Moves
  -- Computer Player Scores by Pieces
  for _, v in pairs(players) do
    v.computeScore(board.pieces)
  end

  declareSituation()
  
  if timerstatus == 'on' then
    players[turn_counter].timer.reset_byoyomi()
  end
  
  -- AI: Also at the bottom of state_play.load()
  if who_won == -1 and turn_counter == ai then
    local from = 0
    local to = 0
    from, to = jang.chooseMove(ai, board.pieces)
    if from > 0 then guiMovePiece(from, to) end
  end
end

--Game: Player Buttons
local function playerSkipTurn(color)
  -- You can only skip your turn if there are no issues
  if engine.softEvaluate(board.points, board.move_history, color) == '' and who_won == -1 then
    for _, v in pairs(board.pieces) do
      if v.char:upper() == 'G' and engine.getColor(v.char) == color then
        guiMovePiece(v.index, v.index) -- General moves to itself
      end
    end
  end
end
local function playerResign(color)
  -- Generals Move Outside the Fortress (For now, I don't care if it eats a piece or not)
  if who_won == -1 then
    for _, v in pairs(board.pieces) do
      if     color == 1 and v.char == 'G' then guiMovePiece(v.index, 79)
      elseif color == 2 and v.char == 'g' then guiMovePiece(v.index, 12)
      end
    end
  end
end


function state_play.load(params)
  menu = newGMenu()
  menu.addButton('Save Game', 52, 15,
    function() state_machine.push(state_savegame, {
          player1 = players[1].name,
          player2 = players[2].name,
          representation = engine.getRepresentation(board.points),
          ai = ai,
          turn_counter = turn_counter,
          move_history = board.move_history
        }
      )
    end
  )
  menu.addButton('Pause Game', 147, 15, function() state_machine.push(state_pause)   end)
  menu.addButton('Flip Board', 242, 15, function() flipBoard()                       end)
  menu.addButton('Exit Game',       432, 15, function() state_machine.change(state_title) end)

  turn_counter = params.turn_counter or 1
  local eval = params.eval or false
  local rep = params.rep or 'thea aeht' .. '4g4' .. '1c5c1' .. 's s s s s' .. '99' .. 'S S S S S' .. '1C5C1' .. '4G4' .. 'THEA AETH'
  ai = params.ai or 0
  timerstatus = params.timerstatus or 'off'
  
  board = newBoard(rep, turn_counter, eval, params.move_history)
  if #board.move_history > 0 then
    local b = newBoard(rep, turn_counter, false, params.move_history)
    for i = #b.move_history, 1, -1 do
      board.move_history[i].rep = engine.getRepresentation(b.points)
      b.undoMove()
    end
  end
  
  gui_pieces = getGuiPieces(board.pieces)
  gui_moves = {}
  selected_index = 0
  
  players = {}
  local player1y = gui_board.flipped and 65 or 395
  local player2y = gui_board.flipped and 395 or 65
  table.insert(players, newPlayer(603, player1y, {name = 'Chu', color = 1, backcolor = {0, 150/255, 0, 0.8}, timerstatus = timerstatus}))
  table.insert(players, newPlayer(603, player2y, {name = 'Han', color = 2, backcolor = {150/255, 0, 0, 0.8}, timerstatus = timerstatus}))
  players[1].menu.buttons[1].fn = function() playerSkipTurn(1) end   
  players[1].menu.buttons[2].fn = function() playerResign(1)   end
  players[2].menu.buttons[1].fn = function() playerSkipTurn(2) end
  players[2].menu.buttons[2].fn = function() playerResign(2)   end
  for _, v in pairs(players) do
    v.computeScore(board.pieces)
  end
  
  declareSituation()
  
  -- AI: (Also at the Bottom of guiMovePiece)
  if who_won == -1 and turn_counter == ai then
    local from = 0
    local to = 0
    from, to = jang.chooseMove(ai, board.pieces)
    if from > 0 then guiMovePiece(from, to) end
  end
end

function state_play.update(dt)
  cur_radius = cur_radius >= max_radius and max_radius or cur_radius + (spd_radius * dt)
  for _, v in pairs(gui_pieces) do
    v.update(dt)
  end
  if timerstatus == 'on' and who_won == -1 then
    players[turn_counter].update(dt)
    if players[turn_counter].timer.ended == true then
      text = 'Time Loss\n' .. players[turn_counter].name .. ' ran out of time thinking of moves.'
      for _, v in pairs(board.pieces) do
        v.moves = {}
      end
      who_won = (turn_counter%2)+1
      table.remove(menu.buttons, 1)
       menu.addButton('Save Match',52, 15, function() state_machine.push(state_savematch, {
        player1 = players[1].name,
        player2 = players[2].name,
        representation = engine.getRepresentation(board.points),
        turn_counter = turn_counter,
        move_history = board.move_history
        })
        end
      )
    menu.addButton('Play Again', 337, 15, function() state_machine.change(state_hansetup, ai) end)

      gui_moves = {}
    end
  end
end

function state_play.draw()
  -- GUI: Setting up a draw order
  local static = {}
  local moving = {}
  local picked_up = 0
  for i, v in pairs(gui_pieces) do
    if     v.picked_up            then picked_up = i
    elseif v.vx ~= 0 or v.vy ~= 0 then table.insert(moving, i)
    else                               table.insert(static, i)
    end
  end

  --Order 1: Non-Moving Pieces
  for _, v in pairs(static) do
    gui_pieces[v].draw()
  end
  --Order 2: Moves
  love.graphics.setColor(1,1,0)
  for _, v in pairs(gui_moves) do
    love.graphics.circle('line', gui_board.points[v].x, gui_board.points[v].y, cur_radius)
  end
  --Order 3: Moving Pieces
  love.graphics.setColor(1,1,1)
  for _, v in pairs(moving) do
    gui_pieces[v].draw()
  end
  --Order 4: The Picked up piece
  if picked_up > 0 then gui_pieces[picked_up].draw() end

  --Order 5: (GUI-from-Internal) Previous Move Indicator
  if board.history_position > 0 then
    -- GUI: Indicate the color that moved previously
    local prev_color = turn_counter == 2 and {0, 120/255, 0, 0.9} or {175/255, 0, 0, 0.7}
    love.graphics.setColor(prev_color)
    
    -- Internal: get the index of the move history from the position
    local index = board.move_history[board.history_position].from    
    love.graphics.rectangle('line',
      gui_board.points[index].x - max_radius,
      gui_board.points[index].y - max_radius,
      max_radius * 2, max_radius * 2,
      3, 3
    )
  end
  
  --Side Bar
  for _, v in pairs(players) do
    v.draw(turn_counter)
  end

  love.graphics.setFont(font)
  love.graphics.printf(text, players[1].x, 250, 188, 'center')
  
  --Menu
  menu.draw()
end

function state_play.keypressed(key)
  if key == 'space' then
    flipBoard()
  end
end

function state_play.mousemoved(x, y, dx, dy)
  players[turn_counter].menu.mouse_over(x, y)
  menu.mouse_over(x, y)
end
function state_play.mousepressed(x, y, button)
  if button == 1 then
    --Player Input First
    players[turn_counter].menu.button_press()
    
    --[[PHASE 1 - GUI: Search if it has pressed on a move--]]
    local selected_move = guiSelectMove(x, y)
    --[[PHASE 2a (If a Move is Selected) - GUI: Search if it has pressed on a piece--]]
    if selected_move > 0 then
      guiMovePiece(selected_index, selected_move)
    --[[PHASE 2b (If No Move is Selected) - GUI: Search if it has pressed on a piece--]]
    else
      if selected_index ~= guiSelectPiece(x, y) then
        selected_index = guiSelectPiece(x, y)
        -- GUI: Reset Radius ONLY if picked up another piece.
        cur_radius = 0
      end
    end
    
    menu.button_press()
  end
end
function state_play.mousereleased(x, y, button)
  if button == 1 then
    --Player Input First
    players[turn_counter].menu.button_release()

    -- GUI: Release every piece
    for i, v in pairs(gui_pieces) do
      if v.index == selected_index then
        -- Same as mousepressed PHASES 1-2a
        local dropped_on_a_move = guiSelectMove(x, y, v.size/2)
        if dropped_on_a_move > 0 then
          guiMovePiece(selected_index, dropped_on_a_move)
          
          -- Forcing them into position because sometimes the piece just flies away if I don't
          v.x = v.tx
          v.y = v.ty
        end
      end
      v.picked_up = false
    end
    
    menu.button_release()
  end
end
