state_position = newState()
local board = {}
local gui_pieces = {}
local gui_index = 0
local gui_radius = 40

local btn_pieces = {}
local btn_index = 0
local btn_radius = 56
local btn_piece = {}

local menu = newGMenu()
local submenu_play = newGSubMenu()

local function flipBoard()
  gui_board.flip()
  for _, v in pairs(gui_pieces) do
    v.setPosition(gui_board.points[v.index].x, gui_board.points[v.index].y)
  end
end

menu.addButton('Save Position', 60, 15, function()
    local items = love.filesystem.getDirectoryItems('')
    for i = #items, 1, -1 do
      if items[i]:sub(-4) ~= '.pos' then
        table.remove(items, i)
      end
    end
    
    if #items < 135 then -- 135 items can only fit in the UI, so I chose that
      state_machine.push(state_saveposition, engine.getRepresentation(board.points))
    end
  end
)
menu.addButton('Play Position', 170, 15, function() submenu_play.visible = true end)
  submenu_play.addButton('As Chu', 143, 38, function()
      state_machine.change(state_play, { rep = engine.getRepresentation(board.points), eval = true})
    end
  )
  submenu_play.addButton('As Han', 195, 38, function()
      state_machine.change(state_play, { rep = engine.getRepresentation(board.points), turn_counter = 2, eval = true })
    end
  )

menu.addButton('Starting Pieces', 280, 15, function()
    board = newBoard('thea aeht4g41c5c1s s s s s99S S S S S1C5C14G4THEA AEHT')
    gui_pieces = getGuiPieces(board.pieces)
  end
)
menu.addButton('Clear Setup', 390, 15, function()
    board = newBoard('94g49999994G')
    gui_pieces = getGuiPieces(board.pieces)
  end
)
menu.addButton('Flip Board', 500, 15, function() flipBoard() end)
menu.addButton('Exit Editor', 610, 15, function() state_machine.change(state_title) end)

local function dropPiece(index, char)
  if board.points[index]:upper() ~= 'G' then -- Don't Drop on General
    --First Three Conditions: Don't Drop Generals and Advisers outside their fortresses
    if (char:upper() == 'G' or char:upper() == 'A') and (engine.getFile(index) < 4 or engine.getFile(index) > 6) then
    elseif (char == 'g' or char == 'a') and index > 24 then
    elseif (char == 'G' or char == 'A') and index < 67 then
    else
      board.points[index] = char
      -- Remove pieces on that index, if ever.
      for i, v in pairs(board.pieces) do
        if v.index == index then
          table.remove(board.pieces, i)
        end
      end
      -- Also remove it in GUI
      for i, v in pairs(gui_pieces) do
        if v.index == index then
          table.remove(gui_pieces, i)
        end
      end
      -- Then insert the new piece in both internal and gui pieces
      table.insert(board.pieces, newPiece(index, char))
      local gui_piece = newGuiPiece(char, index)
      gui_piece.setPosition(gui_board.points[index].x, gui_board.points[index].y)
      table.insert(gui_pieces, gui_piece)
      dropped = true
    end
  end
end

function state_position.load(rep)
  rep = rep or '94g49999994G'
  board = newBoard(rep)
  gui_pieces = getGuiPieces(board.pieces)
  btn_pieces = {}
  table.insert(btn_pieces, newGuiPiece('T',  1)) btn_pieces[ 1].setPosition(655, 350)
  table.insert(btn_pieces, newGuiPiece('H',  2)) btn_pieces[ 2].setPosition(655, 435)
  table.insert(btn_pieces, newGuiPiece('S',  3)) btn_pieces[ 3].setPosition(655, 520)
  table.insert(btn_pieces, newGuiPiece('C',  4)) btn_pieces[ 4].setPosition(735, 350)
  table.insert(btn_pieces, newGuiPiece('E',  5)) btn_pieces[ 5].setPosition(735, 435)
  table.insert(btn_pieces, newGuiPiece('A',  6)) btn_pieces[ 6].setPosition(735, 520)

  table.insert(btn_pieces, newGuiPiece('t',  7)) btn_pieces[ 7].setPosition(655, 100)
  table.insert(btn_pieces, newGuiPiece('h',  8)) btn_pieces[ 8].setPosition(655, 185)
  table.insert(btn_pieces, newGuiPiece('s',  9)) btn_pieces[ 9].setPosition(655, 270)
  table.insert(btn_pieces, newGuiPiece('c', 10)) btn_pieces[10].setPosition(735, 100)
  table.insert(btn_pieces, newGuiPiece('e', 11)) btn_pieces[11].setPosition(735, 185)
  table.insert(btn_pieces, newGuiPiece('a', 12)) btn_pieces[12].setPosition(735, 270)
  
  gui_board.load()
end

function state_position.update(dt)
  -- Selected Piece
  if btn_index > 0 then
    btn_piece.setPosition(love.mouse.getX(), love.mouse.getY())
  end
  -- Picked Up Piece
  for _, v in pairs(gui_pieces) do
    if v.picked_up then
      v.x, v.y = love.mouse.getPosition()
    end
  end
end

function state_position.draw()
  -- Gui Pieces
  local picked_up = 0
  for i, v in pairs(gui_pieces) do
    if v.picked_up then picked_up = i
    else                v.draw()
    end
  end
  
  -- Gui Indicator
  if gui_index > 0 then
    love.graphics.rectangle('line', gui_board.points[gui_index].x - (gui_radius/2), gui_board.points[gui_index].y - (gui_radius/2), gui_radius, gui_radius, 3, 3)
  end
  
    -- Btn Pieces
  for _, v in pairs(btn_pieces) do
    v.draw()
  end
  
  -- Btn Indicator & Drop Piece
  if btn_index > 0 then
    love.graphics.rectangle('line', btn_pieces[btn_index].x - (btn_radius/2), btn_pieces[btn_index].y - (btn_radius/2), btn_radius, btn_radius, 4, 4)
    love.graphics.setColor(1,1,1,0.75)
    btn_piece.draw()
  end
  
  -- Picked Up Piece
  if picked_up > 0 then
    gui_pieces[picked_up].draw()
  end

  -- Menus
  menu.draw()
  if submenu_play.visible then
    submenu_play.draw()
  end
end

function state_position.keypressed(key)
  if key == 'space' then
    flipBoard()
  end
end

function state_position.mousemoved(x, y, dx, dy)
  -- GUI Indicator
  gui_index = 0
  for i, v in pairs(gui_board.points) do
    if getCircleCircleDistance(x, y, 0, v.x, v.y, gui_radius) < 0 then
      gui_index = i
    end
  end

  -- Menus
  menu.mouse_over(x, y)
  if submenu_play.visible then
    submenu_play.mouse_over(x, y)
  end  
end
function state_position.mousepressed(x, y, button)
  if button == 1 then
    -- Btn Pieces (Selecting)
    for i, v in pairs(btn_pieces) do
      if getCircleCircleDistance(x, y, 0, v.x, v.y, btn_radius) < 0 then
        btn_index = i
        btn_piece = newGuiPiece(v.char, 1)
      end
    end
    
    -- Btn Pieces (Dropping on Board)
    if btn_index > 0 and gui_index > 0 then
      dropPiece(gui_index, btn_pieces[btn_index].char)
      
    -- Gui Piece: Pick Up
    elseif gui_index > 0 then
      for _, v in pairs(gui_pieces) do
        if v.index == gui_index then
          v.picked_up = true
        end
      end
    end
    
    -- Menus
    menu.button_press()
    if submenu_play.visible then
      submenu_play.button_press()
    end
  elseif button == 2 then
    if btn_index > 0 then
      btn_index = 0
    elseif gui_index > 0 then
      if board.points[gui_index]:upper() ~= 'G' then -- Don't Remove General
        -- Remove Internal Piece
        board.points[gui_index] = ' '
        for i, v in pairs(board.pieces) do
          if v.index == gui_index then
            table.remove(board.pieces, i)
          end
        end
      
        -- Remove Gui Piece
        for i, v in pairs(gui_pieces) do
          if v.index == gui_index then
            table.remove(gui_pieces, i)
          end
        end
      end
    end
  end
end
function state_position.mousereleased(x, y, button)
  if button == 1 then
    
    local gui_index_drop = 0
    local remove_index = 0
    for i, v in pairs(gui_pieces) do
      if v.picked_up and gui_index > 0 and btn_index == 0 and board.points[gui_index]:upper() ~= 'G' then
        gui_index_drop = i
        remove_index = v.index
      end

      -- Gui Pieces: Release all Picked Up Pieces
      v.setPosition(v.tx, v.ty)
      v.picked_up = false
    end
    
    if gui_index_drop > 0 then
      -- These empty conditions mean "Don't Drop the Generals And Advisers Outside their Fortresses"
      if (gui_pieces[gui_index_drop].char:upper() == 'G' or gui_pieces[gui_index_drop].char:upper() == 'A') and
        (engine.getFile(gui_index) < 4 or engine.getFile(gui_index) > 6) then
      elseif (gui_pieces[gui_index_drop].char == 'g' or gui_pieces[gui_index_drop].char == 'a') and gui_index > 24 then
      elseif (gui_pieces[gui_index_drop].char == 'G' or gui_pieces[gui_index_drop].char == 'A') and gui_index < 67 then
      else
        -- Internal Deletion of Original Location
        board.points[remove_index] = ' '
        for i, v in pairs(board.pieces) do
          if v.index == remove_index then
            table.remove(board.pieces, i)
          end
        end

        -- dropPiece will a new piece, so we have to delete the original piece
        local char = gui_pieces[gui_index_drop].char
        table.remove(gui_pieces, gui_index_drop)        
        dropPiece(gui_index, char)
      end
    end
    -- Menus
    if submenu_play.visible then
      submenu_play.visible = false
      submenu_play.button_release()
    end
    menu.button_release()
  end
end
