state_timersettings = newState()
local font = love.graphics.newFont('font/Bergen Mono Regular.otf', 35)
local minifont = love.graphics.newFont()
local bx = 220
local by = 29
local bw = 300
local bh = 30

local activator = newGMenu()
local menu = newGMenu()

local timerstatus = 'off'
local main = 0
local byoyomi = 0
local periods = 0

local mainhour = 0
local mainminute = 0
local mainsecond = 0

local byominute = 0
local byosecond = 0

local function turn(value)
  if value == 'on' then
    activator.visible = false
    menu.visible = true
    bh = 255
    timerstatus = 'on'
  else
    activator.visible = true
    menu.visible = false
    bh = 40
    timerstatus = 'off'
  end
end
local function mainOutput()
  local hour = mainhour > 9 and mainhour or '0' .. mainhour
  local minute = mainminute > 9 and mainminute or '0' .. mainminute
  local second = mainsecond > 9 and mainsecond or '0' .. mainsecond
  
  return hour .. ':' .. minute .. ':' .. second
end
local function saveSettings()
  local m = (mainhour * 3600) + (mainminute * 60) + mainsecond
  local b = (byominute * 60) + byosecond
  local data = timerstatus .. '\n' ..
               m .. '\n' ..
               b .. '\n' ..
               periods
  love.filesystem.write('timer.cfg', data)
end

activator.addButton('Off', bx + bw/2, by + 20, function() turn('on') end)

menu.addButton('On', bx + bw/2, by + 20, function() turn('off') end)
-- Main Time
menu.addButton('+', bx + 90,   by +  70, function() mainhour   = mainhour   == 10 and 10 or mainhour   + 1 end)
menu.addButton('+', bx + bw/2, by +  70, function() mainminute = mainminute == 59 and  0 or mainminute + 1 end)
menu.addButton('+', bx + 215,  by +  70, function() mainsecond = mainsecond == 59 and  0 or mainsecond + 1 end)
menu.addButton('-', bx + 90,   by + 125, function() mainhour   = mainhour   ==  0 and  0 or mainhour   - 1 end)
menu.addButton('-', bx + bw/2, by + 125, function() mainminute = mainminute ==  0 and 59 or mainminute - 1 end)
menu.addButton('-', bx + 215,  by + 125, function() mainsecond = mainsecond ==  0 and 59 or mainsecond - 1 end)
--Byoyomi Periods
menu.addButton('+', bx + bw/4, by + 180, function() periods = periods == 15 and 15 or periods + 1 end)
menu.addButton('-', bx + bw/4, by + 233, function() periods = periods ==  0 and  0 or periods - 1 end)
--Byoyomi
menu.addButton('+', bx + 195, by + 180, function() byominute = byominute == 59 and  0 or byominute + 1 end)
menu.addButton('+', bx + 255, by + 180, function() byosecond = byosecond == 59 and  0 or byosecond + 1 end)
menu.addButton('-', bx + 195, by + 233, function() byominute = byominute ==  0 and 59 or byominute - 1 end)
menu.addButton('-', bx + 255, by + 233, function() byosecond = byosecond ==  0 and 59 or byosecond - 1 end)

function state_timersettings.load()
  if love.filesystem.getInfo('timer.cfg') then
    local data = {}
    for line in love.filesystem.lines('timer.cfg') do
      table.insert(data, line)
    end
    timerstatus = data[1]
    main = tonumber(data[2])
    byoyomi = tonumber(data[3])
    periods = tonumber(data[4])
  else
    activator.visible = true
    menu.visible = false
    timerstatus = 'off'
    main = 1200
    byoyomi = 40
    periods = 3
  end
  
  turn(timerstatus)
  
  mainhour = math.floor(main/3600)
  mainminute = math.floor((main - mainhour*3600)/60)
  mainsecond = main%60
  
  byominute = math.floor(byoyomi/60)
  byosecond = byoyomi%60
end

local function byoOutput()
  local minute = byominute > 9 and byominute or '0' .. byominute
  local second = byosecond > 9 and byosecond or '0' .. byosecond
  return minute .. ':' .. second
end

function state_timersettings.draw()
  love.graphics.setColor(0,0,100/255,0.75)
  love.graphics.rectangle('fill', bx, by, bw, bh, 3, 3)
  love.graphics.setColor(1,1,1)
  
  if activator.visible then activator.draw() end

  if menu.visible then
    love.graphics.setFont(minifont)
    love.graphics.printf('Main Time',       bx, by + 40, bw, 'center')
    love.graphics.printf('Byoyomi Periods', bx, by + 150, bw/2, 'center')
    love.graphics.printf('Byoyomi Time',    bx + bw/2, by + 150, bw/2, 'center')
    love.graphics.setFont(font)
    love.graphics.printf(mainOutput(), bx, by + 75, bw, 'center')
    love.graphics.printf(periods,      bx, by + 185, bw/2, 'center')
    love.graphics.printf(byoOutput(),  bx + bw/2, by + 185, bw/2, 'center')
    menu.draw()
  end
end

function state_timersettings.mousemoved(x, y, dx, dy)
  if activator.visible then activator.mouse_over(x, y) end
  if menu.visible then menu.mouse_over(x, y) end
end
function state_timersettings.mousepressed(x, y, button)
  if button == 2 or x < bx or x > x + bw or y < by or y > by + bh then
    saveSettings()
    state_machine.pop()
  elseif button == 1 then
    if activator.visible then activator.button_press() end
    if menu.visible then menu.button_press() end
  end
end
function state_timersettings.mousereleased(x, y, button)
  if button == 1 then
    if activator.visible then activator.button_release() end
    if menu.visible then menu.button_release() end
  end
end
